{**********************************************************}
{  WildfireZ Support Functions                             }
{                                                          }
{  wildfireZ version 2.0.1 - 2000.06.14                    }
{  Coded by Geoffrey Michaels                              }
{  email: GeoffreyM@UltratechCreations.com                 }
{                                                          }
{  (c) 1999-2000 Ultratech Creations                       }
{  wildfireZ is a trademark of Ultratech Creations.        }
{                                                          }
{  website: www.UltratechCreations.com                     }
{                                                          }
{  This library is free software; you can redistribute it  }
{  and/or modify it under the terms of the GNU Lesser      }
{  General Public License as published by the Free         }
{  Software Foundation; either version 2.1 of the License, }
{  or (at your option) any later version.                  }
{                                                          }
{  This library is distributed in the hope that it will    }
{  be useful, but WITHOUT ANY WARRANTY; without even the   }
{  implied warranty of MERCHANTABILITY or FITNESS FOR A    }
{  PARTICULAR PURPOSE.  See the GNU Lesser General Public  }
{  License for more details.                               }
{                                                          }
{  You should have received a copy of the GNU Lesser       }
{  General Public License along with this library; if not, }
{  write to the Free Software Foundation, Inc., 59 Temple  }
{  Place, Suite 330, Boston, MA  02111-1307  USA           }
{                                                          }
{**********************************************************}

unit WildfireZSupport;

interface

uses WildfireZDataTypes;

  // exported routines

  // IDL time routines
  function ConvertGMTToIDL(GMTOffset: Integer): Integer; stdcall;
  function ConvertIDLToGMT(IDLOffset: Integer): Integer; stdcall;
  function ConvertToLocalDateTime(UDT: TUnpackedHiresDateTime): TUnpackedHiresDateTime; stdcall;
  function GetIDLOffset(): Integer; stdcall;
  function GetIDLMinutesOffset(): Integer; stdcall;

  // HiresDateTime routines
  function DecodeHiresDateTime(HiresDateTime: THiresDateTime):
    TUnpackedHiresDateTime; stdcall;
  function EncodeHiresDateTime(UnpackedDateTime: TUnpackedHiresDateTime):
    THiresDateTime; stdcall;
  function GetCurrentHiresDateTime(): TUnpackedHiresDateTime; stdcall;
  function IsLocalHiresDateTime(HiresDateTime: THiresDateTime): Boolean; stdcall;

  // Elapsed time count routines
  function GetElapsedTimeCount(): Int64; stdcall;
  function GetElapsedTimeFrequency(): Int64; stdcall;
  function IsElapsedTimeAvailable(): Boolean; stdcall;
  function GetStartupElapsedTime(): Int64; stdcall;
  function GetStartupTime(): Int64; stdcall;

  // misc routines
  function GetRandomNumber(Range: Integer): Integer; stdcall;

implementation

uses SysUtils, Windows, WildfireZKernel;

var
  FDummy: Boolean = False;
  FElapsedTimeAvailable: Boolean = False;
  FElapsedTimeCount: Int64 = 0;
  FElapsedTimeFrequency: Int64 = 0;
  FElapsedTimeStart: Int64 = 0;
  FLocalIDLOffset: Integer = -1;
  FLocalIDLMinutesOffset: Integer = -1;

const
  FMonthDays : array[1..12] of integer = (31,28,31,30,31,30,31,31,30,31,30,31);

// exported routines
function ConvertGMTToIDL(GMTOffset: Integer): Integer; stdcall;
begin
  Result := 0;
  if GetEnvironmentState = esOffline then exit;
  Result := 12 - GMTOffset;
end;

function ConvertIDLTOGMT(IDLOffset: Integer): Integer; stdcall;
begin
  Result := 0;
  if GetEnvironmentState = esOffline then exit;
  Result := 12 - IDLOffset;
end;

function ConvertToLocalDateTime(UDT: TUnpackedHiresDateTime): TUnpackedHiresDateTime; stdcall;
begin
  Result := UDT;
  if GetEnvironmentState = esOffline then exit;
  if (GetIDLMinutesOffset <> UDT.IDLMinutesOffset) then
      Inc(UDT.Minute, GetIDLMinutesOffset - UDT.IDLMinutesOffset);
  if (UDT.Minute < 0) then
    begin
      Inc(UDT.Minute, 60);
      Dec(UDT.Hour, 1);
    end;
  if (UDT.Minute > 59) then
    begin
      Dec(UDT.Minute, 60);
      Inc(UDT.Hour, 1);
    end;
  // adjust for time zone offset
  if (UDT.IDLOffset <> GetIDLOffset) then
    Inc(UDT.Hour, UDT.IDLOffset - GetIDLOffset);
  if (UDT.Hour < 0) then
    begin
      Inc(UDT.Hour, 24);
      Dec(UDT.Day, 1);
      Dec(UDT.DayOfWeek, 1);
    end;
  if (UDT.Hour > 23) then
    begin
      Dec(UDT.Hour, 24);
      Inc(UDT.Day, 1);
      Inc(UDT.DayOfWeek, 1);
    end;
  // adjust day of week
  if (UDT.DayOfWeek < 1) then Inc(UDT.DayOfWeek, 7);
  if (UDT.DayOfWeek > 7) then Dec(UDT.DayOfWeek, 7);
  // adjust day
  if (UDT.Day < 1) then
    begin
      Dec(UDT.Month, 1);
      if (UDT.Month < 1) then
        begin
          Inc(UDT.Month, 12);
          Dec(UDT.Year, 1);
        end;
      UDT.Day := FMonthDays[UDT.Month] - UDT.Day;
      // leap year ?
      if (UDT.Month = 2) AND (UDT.Day = 28) AND ((UDT.Year mod 4) = 0) then
        Inc(UDT.Day, 1);
    end;
  if (UDT.Day > FMonthDays[UDT.Month]) then
    begin
      // leap year ?
      if (UDT.Month = 2) AND (UDT.Day = 29) AND ((UDT.Year mod 4) = 0) then
        begin
          Result := UDT;
          exit;
        end;
      Dec(UDT.Day, FMonthDays[UDT.Month]);
      Inc(UDT.Month, 1);
      if (UDT.Month > 12) then
        begin
          Dec(UDT.Month, 12);
          Inc(UDT.Year, 1);
        end;
    end;
  // mark as local time
  UDT.IDLOffset := GetIDLOffset;
  UDT.IDLMinutesOffset := GetIDLMinutesOffset;
  Result := UDT;
end;

function DecodeHiresDateTime(HiresDateTime: THiresDateTime):
  TUnpackedHiresDateTime; stdcall;
begin
  ZeroMemory(@Result, SizeOf(Result));
  if GetEnvironmentState = esOffline then exit;
  Result.Second := (HiresDateTime.DateTime AND $3F);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 6;
  Result.Minute := (HiresDateTime.DateTime AND $3F);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 6;
  Result.Hour := (HiresDateTime.DateTime AND $1F);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 5;
  Result.DayOfWeek := (HiresDateTime.DateTime AND $07);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 3;
  Result.IDLMinutesOffset := (HiresDateTime.DateTime AND $3F);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 6;
  Result.IDLOffset := (HiresDateTime.DateTime AND $1F);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 5;
  Result.Day := (HiresDateTime.DateTime AND $1F);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 5;
  Result.Month := (HiresDateTime.DateTime AND $0F);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 4;
  Result.Year := (HiresDateTime.DateTime AND $FFF);
// if parity, or new fields added, push existing stuff out of the way
// HiresDateTime.DateTime := HiresDateTime.DateTime shr 12;
  Result.Tick := HiresDateTime.Tick;
end;

function EncodeHiresDateTime(UnpackedDateTime: TUnpackedHiresDateTime):
  THiresDateTime; stdcall;
begin
  SetWFZError(eNoErrors);
  Result.DateTime := 0;
  Result.Tick := 0;
  if GetEnvironmentState = esOffline then exit;
  if (UnpackedDateTime.Year < 1980) then
    begin
      SetWFZError(eWFZK_YearLessThan1980);
      exit;
    end;
  if (UnpackedDateTime.Year > 4095) then
    begin
      SetWFZError(eWFZK_YearMoreThan4095);
      exit;
    end;
  if (UnpackedDateTime.Month < 1) then
    begin
      SetWFZError(eWFZK_MonthLessThan01);
      exit;
    end;
  if (UnpackedDateTime.Month > 12) then
    begin
      SetWFZError(eWFZK_MonthMoreThan12);
      exit;
    end;
  if (UnpackedDateTime.Day < 1) then
    begin
      SetWFZError(eWFZK_DayLessThan01);
      exit;
    end;
  if (UnpackedDateTime.Day > 31) then
    begin
      SetWFZError(eWFZK_DayMoreThan31);
      exit;
    end;
  if (UnpackedDateTime.Month = 2) then
    begin
      if (UnpackedDateTime.Day > 28) AND ((UnpackedDateTime.Year mod 4) <> 0) then
        begin
          SetWFZError(eWFZK_FebruaryMoreThan28Days);
          exit;
        end;
      if (UnpackedDateTime.Day > 29) AND ((UnpackedDateTime.Year mod 4) = 0) then
        begin
          SetWFZError(eWFZK_FebruaryMoreThan29Days);
          exit;
        end;
    end;
  if (UnpackedDateTime.Day > FMonthDays[UnpackedDateTime.Month]) then
    begin
      if (UnpackedDateTime.Month = 4) then
        begin
          SetWFZError(eWFZK_AprilMoreThan30Days);
          exit;
        end;
      if (UnpackedDateTime.Month = 6) then
        begin
          SetWFZError(eWFZK_JuneMoreThan30Days);
          exit;
        end;
      if (UnpackedDateTime.Month = 9) then
        begin
          SetWFZError(eWFZK_SeptemberMoreThan30Days);
          exit;
        end;
      if (UnpackedDateTime.Month = 11) then
        begin
          SetWFZError(eWFZK_NovemberMoreThan30Days);
          exit;
        end;
    end;
  if (UnpackedDateTime.IDLOffset < 0)then
    begin
      SetWFZError(eWFZK_IDLOffsetLessThan00);
      exit;
    end;
  if (UnpackedDateTime.IDLOffset > 24)then
    begin
      SetWFZError(eWFZK_IDLOffsetMoreThan24);
      exit;
    end;
  if (UnpackedDateTime.IDLMinutesOffset < 0) then
    begin
      SetWFZError(eWFZK_IDLMinutesOffsetLessThan00);
      exit;
    end;
  if (UnpackedDateTime.IDLMinutesOffset > 59) then
    begin
      SetWFZError(eWFZK_IDLMinutesOffsetMoreThan59);
      exit;
    end;
  if (UnpackedDateTime.DayOfWeek < 1) then
    begin
      SetWFZError(eWFZK_DayOfWeekLessThan01);
      exit;
    end;
  if (UnpackedDateTime.DayOfWeek > 7) then
    begin
      SetWFZError(eWFZK_DayOfWeekMoreThan07);
      exit;
    end;
  if (UnpackedDateTime.Hour < 1) then
    begin
      SetWFZError(eWFZK_HourLessThan01);
      exit;
    end;
  if (UnpackedDateTime.Hour > 23) then
    begin
      SetWFZError(eWFZK_HourMoreThan23);
      exit;
    end;
  if (UnpackedDateTime.Minute < 1) then
    begin
      SetWFZError(eWFZK_MinuteLessThan01);
      exit;
    end;
  if (UnpackedDateTime.Minute > 59) then
    begin
      SetWFZError(eWFZK_MinuteMoreThan59);
      exit;
    end;
  if (UnpackedDateTime.Second < 1) then
    begin
      SetWFZError(eWFZK_SecondLessThan01);
      exit;
    end;
  if (UnpackedDateTime.Second > 59) then
    begin
      SetWFZError(eWFZK_SecondMoreThan59);
      exit;
    end;
// if parity or new fields, then add, then shl 12
  Result.DateTime := (UnpackedDateTime.Year AND $FFF);
  Result.DateTime := Result.DateTime shl 4;
  Result.DateTime := Result.DateTime OR (UnpackedDateTime.Month AND $0F);
  Result.DateTime := Result.DateTime shl 5;
  Result.DateTime := Result.DateTime OR (UnpackedDateTime.Day AND $1F);
  Result.DateTime := Result.DateTime shl 5;
  Result.DateTime := Result.DateTime OR (UnpackedDateTime.IDLOffset AND $1F);
  Result.DateTime := Result.DateTime shl 6;
  Result.DateTime := Result.DateTime OR (UnpackedDateTime.IDLMinutesOffset AND $3F);
  Result.DateTime := Result.DateTime shl 3;
  Result.DateTime := Result.DateTime OR (UnpackedDateTime.DayOfWeek AND $07);
  Result.DateTime := Result.DateTime shl 5;
  Result.DateTime := Result.DateTime OR (UnpackedDateTime.Hour AND $1F);
  Result.DateTime := Result.DateTime shl 6;
  Result.DateTime := Result.DateTime OR (UnpackedDateTime.Minute AND $3F);
  Result.DateTime := Result.DateTime shl 6;
  Result.DateTime := Result.DateTime OR (UnpackedDateTime.Second AND $3F);
  Result.Tick := UnpackedDateTime.Tick;
end;

function GetCurrentHiresDateTime(): TUnpackedHiresDateTime; stdcall;
var GMTTime, LocalTime: TSystemTime;
begin
  ZeroMemory(@Result, SizeOf(Result));
  if GetEnvironmentState = esOffline then exit;
  GetSystemTime(GMTTime);
  GetLocalTime(LocalTime);
  Result.Year := Integer(GMTTime.wYear);
  Result.Month := Integer(GMTTime.wMonth);
  Result.Day := Integer(GMTTime.wDay);
  Result.IDLOffset := ConvertGMTToIDL(Integer(-(GMTTime.wHour - LocalTime.wHour)));
  Result.IDLMinutesOffset := Integer(LocalTime.wMinute - GMTTime.wMinute);
  Result.DayOfWeek := Integer(GMTTime.wDayOfWeek +1); // adjust range
  Result.Hour := Integer(LocalTime.wHour);
  Result.Minute := Integer(LocalTime.wMinute);
  Result.Second := Integer(LocalTime.wSecond);
  // generate tick
  Result.Tick := Integer(GetElapsedTimeCount);
  // if high frequency counter unavailable, then give dummy result
  if result.Tick = 0 then Result.Tick := 31415926;
end;

function GetElapsedTimeCount(): Int64; stdcall;
begin
  Result := FElapsedTimeCount;
  if GetEnvironmentState = esOffline then exit;
  if FElapsedTimeAvailable = True then
    FDummy := QueryPerformanceCounter(FElapsedTimeCount);
end;

function GetElapsedTimeFrequency(): Int64; stdcall;
begin
  Result := 0;
  if GetEnvironmentState = esOffline then exit;
  Result := FElapsedTimeFrequency;
end;

function GetIDLOffset(): Integer; stdcall;
var tmpDateTime: TUnpackedHiresDateTime;
begin
  Result := 0;
  if GetEnvironmentState = esOffline then exit;
  If FLocalIDLOffset = -1 then
    begin
     tmpDateTime := GetCurrentHiresDateTime;
     FLocalIDLOffset := tmpDateTime.IDLOffset;
     FLocalIDLMinutesOffset := tmpDateTime.IDLMinutesOffset;
    end;
  Result := FLocalIDLOffset;
end;

function GetIDLMinutesOffset(): Integer; stdcall;
var tmpDateTime: TUnpackedHiresDateTime;
begin
  Result := 0;
  if GetEnvironmentState = esOffline then exit;
  If FLocalIDLMinutesOffset = -1 then
    begin
     tmpDateTime := GetCurrentHiresDateTime;
     FLocalIDLOffset := tmpDateTime.IDLOffset;
     FLocalIDLMinutesOffset := tmpDateTime.IDLMinutesOffset;
    end;
  Result := FLocalIDLMinutesOffset;
end;

function GetRandomNumber(Range: Integer): Integer; stdcall;
begin
  Result := 0;
  if GetEnvironmentState = esOffline then exit;
  // NOTE: Delphi can't generate 64-bit random numbers, must use 32-bit
  if Range <=1 then exit;
  Randomize;
  Result := Random(Range);
end;

function GetStartupElapsedTime(): Int64; stdcall;
begin
  Result := 0;
  if GetEnvironmentState = esOffline then exit;
  // calculate time since kernel startup called
  Result := GetElapsedTimeCount - FElapsedTimeStart;
end;

function GetStartupTime(): Int64; stdcall;
begin
  Result := 0;
  if GetEnvironmentState = esOffline then exit;
  Result := FElapsedTimeStart;
end;

function IsElapsedTimeAvailable(): Boolean; stdcall;
begin
  Result := False;
  if GetEnvironmentState = esOffline then exit;
  if FElapsedTimeAvailable = False then
    begin
      FDummy := QueryPerformanceFrequency(FElapsedTimeFrequency);
      if FDummy = True then
        begin
          FElapsedTimeAvailable := True;
          FDummy := QueryPerformanceCounter(FElapsedTimeCount);
          if FElapsedTimeStart = 0 then FElapsedTimeStart := GetElapsedTimeCount; 
        end;
    end;
  Result := FElapsedTimeAvailable;
end;

function IsLocalHiresDateTime(HiresDateTime: THiresDateTime): Boolean; stdcall;
var tmpDateTime: TUnpackedHiresDateTime;
begin
  Result := True;
  if GetEnvironmentState = esOffline then exit;
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 20;
  tmpDateTime.IDLMinutesOffset := (HiresDateTime.DateTime AND $3F);
  HiresDateTime.DateTime := HiresDateTime.DateTime shr 6;
  tmpDateTime.IDLOffset := (HiresDateTime.DateTime AND $1F);
  If (tmpDateTime.IDLOffset = FLocalIDLOffset) OR (tmpDateTime.IDLMinutesOffset =
    FLocalIDLMinutesOffset) then exit;
  Result := False;
end;

end.
